//
//  MMSAttachment.h
//  MMSBase
//
//  Created by John Fox on 10/28/05.
//  Copyright 2005 GroupSmarts, LLC. All rights reserved.
//

#import "MMSPersistentObject.h"
#import "MMSPhoto.h"
#import "MMSPerson.h"

#import <Foundation/Foundation.h>

@interface MMSAttachment : MMSPersistentObject 
{
    NSString *name;
    NSString *notes;
    NSString *ownerID;
    NSString *targetObjectID;
    NSString *targetObjectClassName;
    NSNumber *fileSize;
    NSString *mimeType;
    NSDate *attachmentDate;
    NSDate *lastRevisionDate;
    NSString *checksumString;
    MMSChecksum *checksum;
    NSString *infoDictString;
    NSString *urlString;
    NSURL *url;
    NSString *thumbnailURLString;
    NSURL *thumbnailURL;
    NSString *relativeStoragePath;

    MMSPerson *owner;
    MMSPersistentObject *targetObject;
}

+ (NSArray *)videoFileTypes;
+ (NSArray *)audioFileTypes;
+ (NSArray *)allVideoAttachments;

// 
// Creation methods
//
+ (BOOL)attachmentAtURL:(NSURL *)anURL isUniqueForTargetObject:(MMSPersistentObject *)aTargetObject;

+ (MMSAttachment *)attachmentFromURL:(NSURL *)anURL 
                     forTargetObject:(MMSPersistentObject *)aTargetObject
                               owner:(MMSPerson *)aPerson
                                name:(NSString *)aName;

+ (BOOL)isAudioOnlyQTMoveAtPath:(NSString *)aPath;

//
// Processing
//
+ (NSString *)mimeTypeFromExtension:(NSString *)anExtension;
- (void)processLocalFileAtPath:(NSString *)aPath;

//
// Conveniences
//
- (NSString *)displayName;
- (NSImage *)iconImage;
- (NSString *)ownerName;
+ (NSString *)attachmentsFolderName;
- (NSString *)relativePath;
- (NSString *)pathRelativeToLibraryFolder;
- (NSString *)mediaType;
- (BOOL)isAVFile;

// Library file management support
- (NSURL *)urlFromDBRecord;
- (BOOL)copyAttachmentFileToPath;
- (NSString *)pathForFileInLibrary;

//
// Accessors
//

- (NSString *)name;
- (void)setName:(NSString *)aName;

- (NSString *)notes;
- (void)setNotes:(NSString *)aNotes;

- (NSString *)ownerID;
- (void)setOwnerID:(NSString *)anOwnerID;

- (NSNumber *)fileSize;
- (void)setFileSize:(NSNumber *)aFileSize;

- (NSString *)mimeType;
- (void)setMimeType:(NSString *)aMimeType;

- (NSDate *)attachmentDate;
- (void)setAttachmentDate:(NSDate *)anAttachmentDate;

- (NSDate *)lastRevisionDate;
- (void)setLastRevisionDate:(NSDate *)aLastRevisionDate;

- (NSString *)checksumString;
- (void)setChecksumString:(NSString *)aChecksumString;

- (MMSChecksum *)checksum;
- (void)setChecksum:(MMSChecksum *)aChecksum;

- (NSString *)infoDictString;
- (void)setInfoDictString:(NSString *)anInfoDictString;

- (MMSPerson *)owner;
- (void)setOwner:(MMSPerson *)anOwner;

- (NSString *)relativeStoragePath;
- (void)setRelativeStoragePath:(NSString *)aPath;

- (MMSPersistentObject *)targetObject;
- (void)setTargetObject:(MMSPersistentObject *)aTargetObject;

- (NSString *)targetObjectClassName;
- (void)setTargetObjectClassName:(NSString *)aTargetObjectClassName;

- (NSString *)targetObjectID;
- (void)setTargetObjectID:(NSString *)aTargetObjectID;

- (NSString *)thumbnailURLString;
- (void)setThumbnailURLString:(NSString *)aThumbnailURLString;

- (NSURL *)thumbnailURL;
- (void)setThumbnailURL:(NSURL *)aThumbnailURL;

- (NSString *)urlString;
- (void)setUrlString:(NSString *)anUrlString;

- (NSURL *)url;
- (void)setUrl:(NSURL *)anURL;

@end
