//
//  MMSOrderedCollection.h
//  MMSBase
//
//  Created by John Fox on 4/3/08.
//  Copyright 2008 GroupSmarts, LLC. All rights reserved.
//

#import <Foundation/Foundation.h>

#import "MMSPersistentObject.h"
#import "MMSPerson.h"

@class MMSPhotoCollectionLink;

@interface MMSOrderedCollection : MMSPersistentObject 
{

    NSString *name;
    NSString *notes;
    NSString *ownerID;
    NSDate *creationDate;
    NSDate *lastUpdate;
    NSString *publicationURLString;
    NSURL *publicationURL;
    
    MMSPerson *owner;
    MMSOrderedCollection *parentCollection;
	NSArray *photoCollectionLinks;
	
	NSArray *childCollections;
}

// Creation
+ (MMSOrderedCollection *)newCollectionNamed:(NSString *)aName
								  withPhotos:(NSArray *)aPhotoArray;

+ (MMSOrderedCollection *)newUntitledCollection;

- (NSNumber *)nextAvailableOrderIndex;

//
// Action Methods
//
- (void)addPhoto:(MMSPhoto *)aPhoto;
- (void)removePhoto:(MMSPhoto *)aPhoto;

// Conveniences
+ (NSArray *)rootCollectionsSortedByName;
+ (MMSOrderedCollection *)rootCollection;

- (void)addPhotosInArray:(NSArray *)aPhotoArray;
- (void)addPhoto:(MMSPhoto *)aPhoto;

- (NSInteger)maxOrderIndex;

- (NSArray *)orderedPhotos;
- (NSArray *)photoCollectionLinksFromDB;

- (NSInteger)numberOfChildren;// Returns -1 for leaf nodes
- (MMSOrderedCollection *)childAtIndex:(NSInteger)n;// Invalid to call on leaf nodes

- (void)swapPhotoLink:(MMSPhotoCollectionLink *)firstPhotoLink 
		withPhotoLink:(MMSPhotoCollectionLink *)secondPhotoLink;

- (MMSPhotoCollectionLink *)collectionLinkForPhoto:(MMSPhoto *)aPhoto;

- (MMSPhotoCollectionLink *)collectionLinkForPhotoWithID:(NSString *)aPhotoID;


- (NSNumber *)photoCount;

//
// Flickr Support
//
- (void)mapToFlickrPhotosetID:(NSString *)aPhotosetID;
- (void)removeFlickrPhotosetMapping;

//
// Accessors
//

- (NSArray *)childCollections;
- (void)setChildCollections:(NSArray *)anArray;

- (NSArray *)childCollectionsFromDB;

- (NSString *)name;
- (void)setName:(NSString *)aName;

- (NSString *)notes;
- (void)setNotes:(NSString *)value;

- (NSString *)ownerID;
- (void)setOwnerID:(NSString *)anOwnerID;

- (MMSPerson *)owner;
- (void)setOwner:(MMSPerson *)anOwner;

- (MMSOrderedCollection *)parentCollection;
- (void)setParentCollection:(MMSOrderedCollection *)aParent;

- (NSDate *)creationDate;
- (void)setCreationDate:(NSDate *)aCreationDate;

- (NSDate *)lastUpdate;
- (void)setLastUpdate:(NSDate *)aLastUpdateDate;

- (NSArray *)photoCollectionLinks;
- (void)setPhotoCollectionLinks:(NSArray *)anArray;

- (NSString *)publicationURLString;
- (void)setPublicationURLString:(NSString *)aPublicationURLString;

- (NSURL *)publicationURL;
- (void)setPublicationURL:(NSURL *)aPublicationURL;

@end
