//
//  MMSPersistentObject.h
//  MemoryMiner
//
//  Created by John Fox on 10/8/04.
//  Copyright 2004 MemoryMiner Software. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import <Quartz/Quartz.h>
#import "DataCrux.h"
#import "DC_SearchDefinition.h"
#import "MMSBase.h"
#import "MMSWebServiceHelper.h"

@interface MMSPersistentObject : NSObject 
{
    NSString *objectID;
    NSString *uuid;
	CFUUIDRef uuidRef;
    DataCrux *dataCrux;
    
    NSArray *attachments;
    NSString *keywordMappingString;

    BOOL isSelected;
}

+ (NSString *)dbPath;
+ (DataCrux *)sharedDataCrux;
+ (void)setSharedDataCrux:(DataCrux *)aDataCrux;
+ (DataCrux *)newDataCrux;
+ (DataCrux *)newDataCruxUsingDatabaseAtPath:(NSString *)aDBPath;
+ (MMSPersistentObject *)newObject;
+ (DataCrux *)secondaryDataCrux;
+ (NSString *)memberName;
+ (NSArray *)allObjects;
+ (NSArray *)allObjectsUsingDataCrux:(DataCrux *)aDataCrux;
+ (NSArray *)xmlSerializationsKeys;
+ (BOOL)isAsianLocale;
+ (BOOL)isJapaneseLocale;
+ (BOOL)isEnglishLocale;
+ (NSString *)userLocaleName;

- (DataCrux *)dataCrux;
- (NSString *)archiveString;
- (BOOL)hasAttachments;

//
// Configuration
//
+ (void)performNeededSetup;
+ (void)createLibraryFolderStructureAtPath:(NSString *)aLibraryPath;
+ (void)updateLibraryPathDefaultUsingPath:(NSString *)aPath;
+ (void)loadOrCreateNewLibraryAtPath:(NSString *)aPath;
+ (NSString *)defaultLibraryFolderPath;
+ (BOOL)udateDBAtPath:(NSString *)aDBPath;
- (void)configureUUID;


// Keyword to object mapping support
+ (MMSPersistentObject *)objectWithKeywordMappingString:(NSString *)aString;
+ (NSMutableDictionary *)keywordMappingInfoDict;
+ (NSString *)keywordMappingInfoDictPath;
+ (void)saveChangesToKeywordMappingInfoDict;
+ (BOOL)isUniqueKeywordMappingString:(NSString *)aString;

// Flickr Mapping Support
+ (NSMutableDictionary *)flickrMappingInfoDict;
+ (NSString *)flickrMappingInfoDictPath;
+ (void)saveChangesToFlickrMappingInfoDict;

// 
// Web Export support
//
+ (void)copyFlashResourcesFromTemplateNamed:(NSString *)aTemplateName
                                     toPath:(NSString *)aPath;

+ (void)copyWebExportResourcesFromTemplateAtPath:(NSString *)aTemplatePath
										  toPath:(NSString *)aPath;

+ (NSString *)webExportTemplatePath;

+ (void)updateFlashTemplatesFolder;
+ (void)updateWebTemplatesFolder;
+ (NSString *)templatesPath;
+ (NSString *)thumbnailsPath;
+ (NSString *)mapsPath;
+ (NSString *)flashTemplatesPath;
+ (NSString *)webTemplatesPath;
+ (NSArray *)availableFlashTemplates;
+ (NSDate *)modDateForTemplateNamed:(NSString *)aTemplateName;
+ (NSDate *)modDateForTemplateAtPath:(NSString *)aPath;
+ (void)updateWebTemplatesFolder;

// 
// Fetching
//
+ (id)objectWithValue:(id)anObject forKey:(NSString *)aKey;
+ (id)objectsWithValue:(id)anObject forKey:(NSString *)aKey;

+ (id)objectsWithValue:(id)anObject 
                forKey:(NSString *)aKey
              dataCrux:(DataCrux *)aDataCrux;

+ (id)objectWithValue:(id)anObject 
               forKey:(NSString *)aKey
             dataCrux:(DataCrux *)aDataCrux;

+ (MMSPersistentObject *)objectWithObjectID:(NSString *)anObjectID
                              usingDataCrux:(DataCrux *)aDataCrux;

+ (id)objectsWithNullValueForKey:(NSString *)aKey;
+ (id)objectsWithNonNullValueForKey:(NSString *)aKey;

+ (MMSPersistentObject *)objectWithQualifierDictionary:(NSDictionary *)aQualDict;

+ (NSArray *)objectsWithQualifierDictionary:(NSDictionary *)aQualDict
                                   dataCrux:(DataCrux *)aDataCrux;

+ (NSArray *)objectsWithSQLWhereClause:(NSString *)aSQLWhereClause 
                              dataCrux:(DataCrux *)aDataCrux;

+ (id)qualifierFromDictionary:(NSDictionary *)aQualDict;

+ (NSInteger)objectCountUsingQualifier:(id)aQualifier;
+ (NSArray *)objectsWithQualifierDictionary:(NSDictionary *)aQualDict;

+ (MMSPersistentObject *)objectFromArchiveString:(NSString *)aString;
+ (MMSPersistentObject *)objectWithObjectID:(NSString *)anObjectID;

//
// Persistence
//
+ (void)deleteObject:(id)anObject;
+ (void)deleteObject:(id)anObject
       usingDataCrux:(DataCrux *)aDataCrux;
- (void)saveChanges;
- (void)prepareForDeletion;

//
// Misc
//
+ (NSDateFormatter *)localizedShortDateFormat;

//
// Accessors
//
- (NSArray *)attachments;
- (void)setAttachments:(NSArray *)anAttachments;

- (DataCrux *)dataCrux;
- (void)setDataCrux:(DataCrux *)aDataCrux;

- (NSString *)keywordMappingString;
- (void)setKeywordMappingString:(NSString *)aString;

- (NSString *)objectID;
- (void)setObjectID:(NSString *)anObjectID;

- (BOOL)isSelected;
- (void)setIsSelected:(BOOL)flag;

- (NSString *)uuid;
- (void)setUuid:(NSString *)value;

//
// Conveniences
//
+ (MMSPersistentObject *)newOrExistingObjectUsingDataDict:(NSDictionary *)aDataDict;
- (BOOL)isNewObject;
- (void)handleNewObjectCreation;
- (void)takeValuesFromDictionary:(NSDictionary *)aDictionary;
+ (NSString *)createUUIDString;
- (NSArray *)fetchAttachments;

//
// Library Management
//
+ (NSString *)libraryFolderPath;
+ (NSString *)fetchLibraryFolderPath;
+ (void)setLibraryFolderPath:(NSString *)aLibraryPath;
+ (NSString *)libraryFolderPath;
+ (NSString *)attachmentFolderPathInLibrary;
+ (NSString *)libraryInfoPath;
+ (NSMutableDictionary *)libraryInfoDict;
+ (void)updateLibraryInfoAsSelfContained;
+ (BOOL)saveLibraryInfoDict;
+ (NSString *)currentLibraryVersionString;
+ (BOOL)libraryIsSelfContained;
+ (void)validateLibraryInfo;
+ (void)createAndStoreLibraryInfoDict;
+ (void)perfomNeededUpdateForDBAtPath:(NSString *)aDBPath;

+ (NSString *)quickLookFolderPath;
+ (NSString *)quickLookThumbnailPath;
+ (NSString *)quickLookPreviewPath;

// 
// XML Serialization
//
+ (NSString *)xmlEntityName;
+ (NSString *)xmlPluralEntityName;
- (NSString *)xmlEntityOpener;
- (NSString *)xmlDescriptionWithIndentCount:(NSInteger)anIndentCount;
- (NSString *)xmlDescriptionForAttributeKeys:(NSArray *)aKeyArray
                             withIndentCount:(NSInteger)anIndentCount;
- (NSString *)xmlDescriptionForAttributeKeys:(NSArray *)aKeyArray
                             withIndentCount:(NSInteger)anIndentCount
                                    delegate:(id)aDelegate;

- (NSString *)nonNilValueForKey:(NSString *)aKey;

// 
// Export
//
- (void)copyAttachments:(NSArray *)anAttachmentArray 
				 toPath:(NSString *)aBasePath;

- (void)copyAttachmentsToPath:(NSString *)aBasePath;

//
// MemoryMiner Web Services Support
+ (MMSWebServiceHelper *)sharedWebServiceHelper;
+ (NSString *)webServiceAuthString;
- (NSMutableDictionary *)wsDataDict;
@end

#define locationObjectIDString  @"MMSLocationObjectIDString"
#define personObjectIDString @"MMSPersonObjectIDString"
#define persistentObjectArchiveString @"MMSPersistentObjectArchiveString"
#define persistentObjectArchiveStringArray @"MMSPersistentObjectArchiveStringArray"
#define searchFilterAddedNotification @"MMSSearchFilterAddedNotification"
#define searchFilterRemovalNeededNotification @"MMSSearchFilterRemovalNeedeNotification"
#define searchFilterChainClearedNotification @"MMSSearchFilterChainClearedNotification"
#define attachmentChangeNotification @"MMSAttachmentChangeNotification"
#define newLibraryDidLoadNotification @"MMSNewLibraryDidLoadNotification"
#define willLoadLibraryNotification @"MMSWillLoadLibraryNotification"
#define didLoadAllPeopleNotification @"MMSDidLoadAllPeopleNotification"
#define didLoadAllLocationsotification @"MMSDidLoadAllLocationsNotification"
#define didSavePhotoNotification @"MMSDidSavePhotoNotification"
#define didDeletePhotoNotification @"MMSDidDeletePhotoNotification"

#define MMSLocalizedFrameworkString(key, comment) NSLocalizedStringFromTableInBundle(key, @"Localizable", [NSBundle bundleForClass:[MMSPersistentObject class]], comment)

