@import <AppKit/CPView.j>
@import "MKMapItem.j"
@import "MKMapView.j"
@import "MKLocation.j"

@implementation MKPolyline : MKMapItem
{
    CPArray     _locations     @accessors(property=locations);
    CPString    _colorCode     @accessors(property=colorCode);
    int         _lineStroke    @accessors(property=lineStroke);
	MKMapView	_mapView	   @accessors(property=mapView);
	
	Polyline	_googlePolyline	@accessors(property=polyline);
}

+ (MKPolyline)polyline
{
    return [[MKPolyline alloc] init];
}

- (id)init 
{
    return [self initWithLocations:nil];
}

- (id)initWithLocations:(CPArray)someLocations
{
    if (self = [super init]) {
        _locations = someLocations;
        _colorCode = @"#ff0000";
        _lineStroke = 5;
		_googlePolyline = nil;
    }
	
    return self;
}

- (void)addLocation:(MKLocation)aLocation {
    if (!_locations) {
        _locations = [[CPArray alloc] init];
    }
    
    [_locations addObject:aLocation];
}

- (Polyline)googlePolyline 
{
    if (_locations && !_googlePolyline) 
	{
		var gm = [MKMapView gmNamespace];
		var locEnum = [_locations objectEnumerator];
		var loc = nil
		var lineCoordinates = [];
		while (loc = [locEnum nextObject]) {
			lineCoordinates.push([loc googleLatLng]);
        }

		_googlePolyline = new gm.Polyline(lineCoordinates, _colorCode, _lineStroke);
	}
    
	return _googlePolyline;
}

- (void)refreshGooglePolyline
{
	if (_googlePolyline && _mapView && [_mapView mapReady])
	{
		[_mapView gMap].removeOverlay(_googlePolyline);
	}
		
	_googlePolyline = nil;
}

/*
- (Polyline)googlePolyline {
    if (_locations) {
        var gm = [MKMapView gmNamespace];
        var locEnum = [_locations objectEnumerator];
        var loc = nil
        var lineCoordinates = [];
        while (loc = [locEnum nextObject]) {
            lineCoordinates.push([loc googleLatLng]);
        }
        
        return new gm.Polyline(lineCoordinates, _colorCode, _lineStroke);
    }
    
    return nil;
}
*/

- (void)addToMapView:(MKMapView)aMapView
{
    var googleMap = [aMapView gMap];
    googleMap.addOverlay([self googlePolyline]);
	
	[[aMapView polylineArray] addObject:self];
	
	//log('MKPolyline [mapView polylineArray] is now ' +[aMapView polylineArray]);
	
	_mapView = aMapView;
}

- (void)setHidden:(BOOL)isHidden
{
	if (isHidden)
	{
		[self googlePolyline].hide();
	}
	else
	{
		[self googlePolyline].show();
	}
}

@end

